package kemit.ssl;

/**
 * JXSSLSocketFactory.java
 */

import java.io.*;
import java.net.*;
import javax.net.SocketFactory;
import javax.net.ssl.SSLSocketFactory;

import java.security.*;
import java.awt.*;

import javax.net.ssl.*;   

public class KemitSSLSocketFactory extends SSLSocketFactory {

    private static SSLSocketFactory factory = null;
    private static KemitSSLSocketFactory default_factory = null;
    private static KeyStore clientKeystore;
    private static final String DEFAULT_KEYSTORE_TYPE = "JKS";
    private static ClassLoader myClassLoader = null;


    public static void setClassLoader(ClassLoader newLoader) {
        myClassLoader = newLoader;
    }

    private static ClassLoader getClassLoader() {
        if (myClassLoader == null) myClassLoader = ClassLoader.getSystemClassLoader();

        return myClassLoader;
    }


    public static void setDebug(boolean status)    {

        if (status == true)
            System.setProperty("javax.net.debug", "ssl,handshake,verbose");
        else
        {
            System.setProperty("javax.net.debug", " ");
        }
    }

    
    public static void init(String caKeystoreFile, 
                            String clientKeystoreFile,
                            char[] caPassphrase, 
                            char[] clientPassphrase,
                            String caKeystoreType, 
                            String clientKeystoreType) throws GeneralSecurityException, IOException    {
        
        boolean usingSASL = false;   // whether we are using client-authenticated SSL
        checkFileSanity(caKeystoreFile, clientKeystoreFile, clientPassphrase);
        
        if ((clientPassphrase!=null) && (clientPassphrase.length>0) && (clientKeystoreFile != null)) usingSASL = true;
        if (caKeystoreFile == null && clientKeystoreFile != null) caKeystoreFile = clientKeystoreFile;

        if (caKeystoreType == null) caKeystoreType = DEFAULT_KEYSTORE_TYPE;

        SSLContext sslctx = setSSLContextProtocol();

        KeyManagerFactory clientKeyManagerFactory = null;
        TrustManagerFactory caTrustManagerFactory = null;
        KeyStore caKeystore = null;
        KeyManager[] clientKeyManagers = null;

        if (usingSASL) {
            if (clientKeystoreType == null) clientKeystoreType = DEFAULT_KEYSTORE_TYPE;
            clientKeystore = KeyStore.getInstance(clientKeystoreType); // key manager key store
            if (clientKeystoreFile != null) clientKeystore.load(new FileInputStream(clientKeystoreFile), clientPassphrase);
            clientKeyManagerFactory = KeyManagerFactory.getInstance("SunX509");
            clientKeyManagerFactory.init(clientKeystore, clientPassphrase);
            clientKeyManagers = clientKeyManagerFactory.getKeyManagers();
        }
        else { clientKeystore = null; }

        caKeystore = KeyStore.getInstance(caKeystoreType);
        if (caKeystoreFile != null) { caKeystore.load(new FileInputStream(caKeystoreFile), caPassphrase);}
        String defaultTrustAlgorithm = TrustManagerFactory.getDefaultAlgorithm();
        caTrustManagerFactory = TrustManagerFactory.getInstance(defaultTrustAlgorithm);
        caTrustManagerFactory.init(caKeystore);
        TrustManager[] caTrustManagers = caTrustManagerFactory.getTrustManagers();
        caTrustManagers = KemiTrustManager.convert(caTrustManagers, caKeystore, caKeystoreFile, caPassphrase, caKeystoreType);
        TrustManager[] trustedServerAndCAManagers = caTrustManagers;
        
        sslctx.init(clientKeyManagers, trustedServerAndCAManagers, null);

        factory = sslctx.getSocketFactory();

        synchronized(KemitSSLSocketFactory.class) { default_factory = new KemitSSLSocketFactory();}
    }

    private static SSLContext setSSLContextProtocol() throws NoSuchAlgorithmException {
        SSLContext sslctx;

        String protocol = System.getProperty("sslversion", "TLS"); // TLS for java 1.4
        if (!"TLS".equals(protocol))
            System.out.println("SECURITY WARNING: Using non-standard ssl version: '" + protocol + "'");
        sslctx = SSLContext.getInstance(protocol);
        return sslctx;
    }


    private static void checkFileSanity(String caKeystoreFile, String clientKeystoreFile, char[] clientPassphrase) throws SSLException {
        if (clientKeystoreFile == null && caKeystoreFile == null)
            throw new SSLException("SSL Initialisation error: No valid keystore files available.");

        if (caKeystoreFile != null)
            if (new File(caKeystoreFile).exists() == false)
                throw new SSLException("SSL Initialisation error: file '" + caKeystoreFile + "' does not exist.");

        if (clientKeystoreFile != null && clientPassphrase != null)
            if (new File(clientKeystoreFile).exists() == false)
                throw new SSLException("SSL Initialisation error: file '" + clientKeystoreFile + "' does not exist.");
    }


  // DEBUG PRINT CODE - don't remove, can be quite usefull...
/*
        KeyManager[] myKM = new KeyManager[keyManagers.length];
        for (int i=0; i<keyManagers.length; i++)
        {
            myKM[i] = new MyX509KeyManager((X509KeyManager)keyManagers[i]);
        }

        TrustManager[] myTM = new TrustManager[trustManagers.length];
        for (int i=0; i<trustManagers.length; i++)
        {
            myTM[i] = new MyX509TrustManager((X509TrustManager)trustManagers[i]);
        }

        System.out.println("Number of Keymanagers = " + myKM.length);
        if (myKM.length >=1)
        {
            KeyManager bloop = myKM[0];
            if (bloop == null) System.out.println("Bloop is Null???!");
            System.out.println("bloop is a " + bloop.getClass());
            if (bloop instanceof X509KeyManager)
            {
                System.out.println("bloop is X509KeyManager!");
                String[] clients = ((X509KeyManager)bloop).getClientAliases("SunX509", null);
                System.out.println("Num clients = " + clients.length);
                for (int i=0; i<clients.length; i++)
                    System.out.println("client: " + i + " = " + clients[i]);
            }
        }


        System.out.println("Number of Trustmanagers = " + myTM.length);
        if (myTM.length >=1)
        {
            TrustManager bloop = myTM[0];
            if (bloop == null) System.out.println("Bloop is Null???!");
            System.out.println("bloop is a " + bloop.getClass());
            if (bloop instanceof X509TrustManager)
            {
                System.out.println("bloop is X509TrustManager!");
               ((X509TrustManager)bloop).getAcceptedIssuers();
            }
        }
*/

    /**
     * Constructor
     */
    public KemitSSLSocketFactory()    {
    }


    public static SocketFactory getDefault() {
        synchronized(KemitSSLSocketFactory.class) {
            if (default_factory == null) default_factory = new KemitSSLSocketFactory();
        }
        return default_factory;
    }


    public static KeyStore getClientKeyStore() {
            return clientKeystore;
    }

    public Socket createSocket(String host, int port) throws IOException, UnknownHostException {
        Socket socket=factory.createSocket(host, port);
        socket.setSoTimeout(3000);
        return socket;
    }

    public Socket createSocket(InetAddress host, int port) throws IOException, UnknownHostException {
        Socket socket=factory.createSocket(host, port);
        socket.setSoTimeout(3000);
        return socket;
    }


    public Socket createSocket(InetAddress host, int port,InetAddress client_host, int client_port) throws IOException, UnknownHostException {
        Socket socket=factory.createSocket(host, port, client_host, client_port);
        socket.setSoTimeout(3000);
        return socket;        
    }

    public Socket createSocket(String host, int port,InetAddress client_host, int client_port)throws IOException, UnknownHostException {
        Socket socket=factory.createSocket(host, port, client_host, client_port);
        socket.setSoTimeout(3000);
        return socket;                
    }

    public Socket createSocket(Socket socketO, String host, int port, boolean autoclose) throws IOException, UnknownHostException{
        Socket socket=factory.createSocket(socketO, host, port, autoclose);
        socket.setSoTimeout(3000);
        return socket;                        
    }

    public String[] getDefaultCipherSuites() {
	    return factory.getDefaultCipherSuites();
    }

    public String[] getSupportedCipherSuites() {
    	return factory.getSupportedCipherSuites();
    }

   
}